(***********************************************************************)
(*                                                                     *)
(*                          CamlP3l                                    *)
(*                                                                     *)
(* (C) 2004-2007                                                       *)
(*             Roberto Di Cosmo (dicosmo@dicosmo.org)                  *)
(*             Zheng Li (li@pps.jussieu.fr)                            *)
(*             Pierre Weis (Pierre.Weis@inria.fr)                      *)
(*             Francois Clement (Francois.Clement@inria.fr)            *)
(*                                                                     *)
(* Based on original Ocaml P3L System                                  *)
(* (C) 1997 by                                                         *)
(*             Roberto Di Cosmo (dicosmo@ens.fr)                       *)
(*             Marco Danelutto (marcod@di.unipi.it)                    *)
(*             Xavier Leroy  (Xavier.Leroy@inria.fr)                   *)
(*             Susanna Pelagatti (susanna@di.unipi.it)                 *)
(*                                                                     *)
(* This program is free software; you can redistribute it and/or       *)
(* modify it under the terms of the GNU Library General Public License *)
(* as published by the Free Software Foundation; either version 2      *)
(* of the License, or (at your option) any later version.              *)
(*                                                                     *)
(* This program is distributed in the hope that it will be useful,     *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of      *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *)
(* GNU Library General Public License for more details.                *)
(*                                                                     *)
(***********************************************************************)

(* $Id: config.ml,v 1.3 2007/02/16 16:10:20 fclement Exp $ *)

type graphical_output = Window | Postscript | No;;

let graphical_output_of_string = function
| "window" -> Window
| "ps" | "postscript" -> Postscript
| "no" | "none" -> No
| s -> failwith (Printf.sprintf "Config: Unknown graphical output %s" s)
and string_of_graphical_output = function
| Window -> "window"
| Postscript -> "postscript"
| No -> "none";;

type configuration = {
    rdec_config_filename : string;
    result_basename : string;
    graphical_output : graphical_output;
    outer_itermax : int;
    inner_itermax : int;
    display_step : int;
    a_while : int;
  };;

let configuration_of s1 s2 s3 i1 i2 i3 i4 =
  {rdec_config_filename = s1;
   result_basename = s2;
   graphical_output = graphical_output_of_string s3;
   outer_itermax = i1;
   inner_itermax = i2;
   display_step = i3;
   a_while = i4};;

let parse_configuration_file fname =
  let ib =
    try Scanf.Scanning.from_file fname
    with Sys_error s -> prerr_endline s; exit 6 in
  Scanf.bscanf ib
    "rdec_config_filename = %S;
     result_basename = %S;
     graphical_output = %S;
     outer_itermax = %d;
     inner_itermax = %d;
     display_step = %d;
     a_while = %d;\n"
    configuration_of;;

let print_configuration_file ob cfg =
  Printf.bprintf ob "\
     rdec_config_filename = %S;\n\
     result_basename = %S;\n\
     graphical_output = %S;\n\
     outer_itermax = %d;\n\
     inner_itermax = %d;\n\
     display_step = %d;\n\
     a_while = %d;\n"
    cfg.rdec_config_filename
    cfg.result_basename
    (string_of_graphical_output cfg.graphical_output)
    cfg.outer_itermax
    cfg.inner_itermax
    cfg.display_step
    cfg.a_while
;;

let string_of_configuration_file cfg =
  let ob = Buffer.create 1024 in
  print_configuration_file ob cfg;
  Buffer.contents ob;;
