(***********************************************************************)
(*                                                                     *)
(*                          CamlP3l                                    *)
(*                                                                     *)
(* (C) 2004-2007                                                       *)
(*             Roberto Di Cosmo (dicosmo@dicosmo.org)                  *)
(*             Zheng Li (li@pps.jussieu.fr)                            *)
(*             Pierre Weis (Pierre.Weis@inria.fr)                      *)
(*             Francois Clement (Francois.Clement@inria.fr)            *)
(*                                                                     *)
(* Based on original Ocaml P3L System                                  *)
(* (C) 1997 by                                                         *)
(*             Roberto Di Cosmo (dicosmo@ens.fr)                       *)
(*             Marco Danelutto (marcod@di.unipi.it)                    *)
(*             Xavier Leroy  (Xavier.Leroy@inria.fr)                   *)
(*             Susanna Pelagatti (susanna@di.unipi.it)                 *)
(*                                                                     *)
(* This program is free software; you can redistribute it and/or       *)
(* modify it under the terms of the GNU Library General Public License *)
(* as published by the Free Software Foundation; either version 2      *)
(* of the License, or (at your option) any later version.              *)
(*                                                                     *)
(* This program is distributed in the hope that it will be useful,     *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of      *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *)
(* GNU Library General Public License for more details.                *)
(*                                                                     *)
(***********************************************************************)

(* $Id: config.mli,v 1.3 2007/02/16 16:10:20 fclement Exp $ *)

(**
   Parsing a configuration file for the coupling program.
   
   A configuration file must contain (in that order):
   - the name of the file defining the regular decomposition
   - the basename for numerical files
   - the graphical output type
   - the maximum number of iterations for the domain decomposition algorithm
   - the maximum number of iterations for the local subproblems
   - the refreshment step for the ascii and graphical displays
   - the number of seconds to sleep before terminating
   
 *)

type graphical_output = Window | Postscript | No;;
(** The type of graphical outputs. *)

val graphical_output_of_string : string -> graphical_output;;
val string_of_graphical_output : graphical_output -> string;;
(** Conversion functions. *)

type configuration = {
    rdec_config_filename : string;
    result_basename : string;
    graphical_output : graphical_output;
    outer_itermax : int;
    inner_itermax : int;
    display_step : int;
    a_while : int;
  };;
(** The type of configurations. *)

val parse_configuration_file : string -> configuration;;
(** Parsing a configuration file. *)

val print_configuration_file : Buffer.t -> configuration -> unit;;
(** Printing a configuration file. *)

val string_of_configuration_file : configuration -> string;;
(** string of a configuration file. *)
