(***********************************************************************)
(*                                                                     *)
(*                          CamlP3l                                    *)
(*                                                                     *)
(* (C) 2004-2007                                                       *)
(*             Roberto Di Cosmo (dicosmo@dicosmo.org)                  *)
(*             Zheng Li (li@pps.jussieu.fr)                            *)
(*             Pierre Weis (Pierre.Weis@inria.fr)                      *)
(*             Francois Clement (Francois.Clement@inria.fr)            *)
(*                                                                     *)
(* Based on original Ocaml P3L System                                  *)
(* (C) 1997 by                                                         *)
(*             Roberto Di Cosmo (dicosmo@ens.fr)                       *)
(*             Marco Danelutto (marcod@di.unipi.it)                    *)
(*             Xavier Leroy  (Xavier.Leroy@inria.fr)                   *)
(*             Susanna Pelagatti (susanna@di.unipi.it)                 *)
(*                                                                     *)
(* This program is free software; you can redistribute it and/or       *)
(* modify it under the terms of the GNU Library General Public License *)
(* as published by the Free Software Foundation; either version 2      *)
(* of the License, or (at your option) any later version.              *)
(*                                                                     *)
(* This program is distributed in the hope that it will be useful,     *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of      *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *)
(* GNU Library General Public License for more details.                *)
(*                                                                     *)
(***********************************************************************)

(* $Id: gnuplot.ml,v 1.2 2007/02/06 15:15:51 fclement Exp $ *)

open Config;;
open Rdec2;;

(* The gnuplot version number is set by HTMLC from the knowledge got by an
   external shell script. *)
let gnuplot_version_number = 4.0;;

let check_graphical_output = function
| No -> ()
| _ as go -> 
  if gnuplot_version_number <= 0. then begin
    prerr_endline
      (Printf.sprintf
         "Graphical output %s needs Gnuplot." (string_of_graphical_output go));
    exit 4 end;;

(* The syntax of some gnuplot commands depends on the version number *)
let unset, pointsize =
  let us, ps =
    if gnuplot_version_number >= 4.0 then
      (fun s -> Printf.sprintf "\nunset %s" s),
      (fun () -> Printf.sprintf "")
    else
      (fun s -> Printf.sprintf "\nset no%s" s),
      (fun () -> Printf.sprintf "pointsize 0.5") in
  (fun () -> us),
  (fun () -> ps);;

(* Generating the gnuplot command *)

let make_isosample () (nsbdx, nsbdy) =
  Printf.sprintf "set isosample %d, %d" (nsbdx + 1) (nsbdy + 1)

and make_term_or_title () (output, res, iter) =
  match output with
  | Postscript ->
      Printf.sprintf
        "%a%a%a%a\
        \nset terminal postscript eps color solid\
        \nset output '%s%d.eps'"
        unset "xtics"
        unset "ytics"
        unset "ztics"
        unset "border"
        res iter
  | _ -> Printf.sprintf "\nset title 'iter = %d'" iter

and make_splot () (res, nsbd) =
  let s = ref "\nsplot [0:1][0:1][0:0.07]" in
  let to_do n =
    let resn = Printf.sprintf "%s%d" res n in
    if not (Sys.file_exists resn) then
      begin Printf.eprintf "File %s not found!\n" resn; flush stderr end;
    let file = Printf.sprintf " '%s'" resn in
    let style =
      Printf.sprintf " notitle with points pointtype 7%a," pointsize () in
    s := !s ^ file ^ style in
  for_all_subdomains nsbd to_do;
  !s

and make_reference_surface () output =
  let f, wl = "x*(1-x)*y*(1-y)", "with lines" in
  match output with
  | Postscript -> Printf.sprintf " %s notitle %s linetype 9 linewidth 4\n" f wl
  | _ -> Printf.sprintf " %s %s\n" f wl;;

let make_plot_command output res iter nsbd =
  Printf.sprintf "%a%a%a%a"
    make_isosample nsbd
    make_term_or_title (output, res, iter)
    make_splot (res, nsbd)
    make_reference_surface output;;
