(***********************************************************************)
(*                                                                     *)
(*                          CamlP3l                                    *)
(*                                                                     *)
(* (C) 2004-2007                                                       *)
(*             Roberto Di Cosmo (dicosmo@dicosmo.org)                  *)
(*             Zheng Li (li@pps.jussieu.fr)                            *)
(*             Pierre Weis (Pierre.Weis@inria.fr)                      *)
(*             Francois Clement (Francois.Clement@inria.fr)            *)
(*                                                                     *)
(* Based on original Ocaml P3L System                                  *)
(* (C) 1997 by                                                         *)
(*             Roberto Di Cosmo (dicosmo@ens.fr)                       *)
(*             Marco Danelutto (marcod@di.unipi.it)                    *)
(*             Xavier Leroy  (Xavier.Leroy@inria.fr)                   *)
(*             Susanna Pelagatti (susanna@di.unipi.it)                 *)
(*                                                                     *)
(* This program is free software; you can redistribute it and/or       *)
(* modify it under the terms of the GNU Library General Public License *)
(* as published by the Free Software Foundation; either version 2      *)
(* of the License, or (at your option) any later version.              *)
(*                                                                     *)
(* This program is distributed in the hope that it will be useful,     *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of      *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *)
(* GNU Library General Public License for more details.                *)
(*                                                                     *)
(***********************************************************************)

(* $Id: my_data.ml,v 1.3 2007/07/13 13:49:22 fclement Exp $ *)

open Config;;
open Rdec2;;

type subdomain =
    {config_filename : string;
     config : configuration;
     rdec : regular_decomposition;
     num : int;
     iter : int;
     bd : boundary;
     err : float * float};;

let zero_length_error () =
  invalid_arg "My_data: empty stream item"
and constant_field_error s =
  failwith (Printf.sprintf "My_data: field %s should be constant" s)
and same_length_error () =
  invalid_arg "My_data: inconsistent lengths";;

let check_length v =
  let n = Array.length v in
  if n = 0 then zero_length_error ();;

(* Extraction functions *)

let get_unique get s v =
  check_length v;
  let res = get v.(0) in
  if Uarray.for_all (fun x -> get x = res) v then res else
  constant_field_error s;;

let get_config_filename v =
  get_unique (fun subd -> subd.config_filename) "config_filename" v
and get_config v = get_unique (fun subd -> subd.config) "config" v
and get_rdec v = get_unique (fun subd -> subd.rdec) "rdec" v
and get_nums v = Array.map (fun subd -> subd.num) v
and get_iter v = get_unique (fun subd -> subd.iter) "iter" v
and get_bds v = Array.map (fun subd -> subd.bd) v
and get_errs v = Array.map (fun subd -> subd.err) v;;

(* Insertion function *)

let set_bds bds v =
  Uarray.array_seti (fun i subd -> {subd with bd = bds.(i)}) v;;
