(***********************************************************************)
(*                                                                     *)
(*         Luc Maranget, projet Moscova, INRIA Rocquencourt            *)
(*                                                                     *)
(*  Copyright 2007 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  This file is distributed under the terms of the   *)
(*  GNU Public License version 2, http://www.gnu.org/licenses/gpl.txt  *)
(*                                                                     *)
(***********************************************************************)

let print_DEBUG = print_endline

let default_host = Unix.gethostname()
let default_port = 12345

let parse_host_port s =
  try
    let colon = String.index s ':' in
    let h = String.sub s 0 colon
    and p = String.sub s (colon+1) (String.length s - colon -1) in
    let host = if h <> "" then h else default_host in
    let port = if p <> "" then int_of_string p else default_port in
    (host, port)
  with
  | Not_found -> (s, default_port)
  | Failure _ -> raise (Arg.Bad ("bad port specification " ^ s))
  
let get_addr h =
  try
    (Unix.gethostbyname h).Unix.h_addr_list.(0)
  with Not_found ->
    Printf.eprintf "Not a valid hostname: %s\n" h ;
    exit 2     

let sockaddr h p = Unix.ADDR_INET (get_addr h, p)

let rec get_site n addr =
  try Join.Site.there addr with
  | Join.Exit -> exit 0
  | Failure _ as e ->
      if n > 1 then begin
	Thread.delay 0.5 ; get_site (n-1) addr
      end else raise e

let rec lookup ns key =
  try
    let r = Join.Ns.lookup ns key in
    r
  with _ -> Thread.delay 0.5 ; lookup ns key
