module type Type = sig
  type 'a stream
  type 'a t = 'a stream

  (** Construction functions. *)
  val snil : 'a stream
  val scons : 'a -> 'a stream -> 'a stream
  val singleton : 'a -> 'a stream
  (** [singleton x] returns the stream that has only [x] as element. *)

  val head : 'a stream -> 'a
  (** [head s] returns the first element of stream [s].
      Raises [End_of_stream], if [s] is empty. *)

  val head_tail : 'a stream -> 'a * 'a stream

  val of_fun : (unit -> 'a) -> 'a stream

  exception End_of_stream

  val of_list : 'a list -> 'a stream

  val to_list : 'a stream -> 'a list

  val of_vect : 'a array -> 'a stream

  val to_vect : 'a stream -> 'a array

  (** Merges two lazy stream into a single stream with intermixed elements from
      both streams in a fair way (fairly peeks up alternatively elements from
      both streams). *)
  val join : ('a stream * 'a stream) -> 'a stream

  (** Splits a stream into two. *)
  val fork : 'a stream -> ('a stream * 'a stream)

  val iter : ('a -> unit) -> 'a stream -> unit
  val map : ('a -> 'b) -> 'a stream -> 'b stream
  val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b stream -> 'a

end
  
