type 'a stream = {  hd : unit -> 'a }

type 'a t = 'a stream

exception End_of_stream

let snil = { hd = (fun () -> raise End_of_stream) } 


let scons x s =
  let body =  ref (fun () -> assert false) in
  let s_hd = s.hd in
  let f () =
    body := s_hd;
    x
  in
  body := f;
  { hd = (fun () -> !body ())}


let singleton x = scons x snil

let head s = s.hd()
let head_opt s = 
  try Some (s.hd())
  with End_of_stream -> None 

let head_tail s = 
  let x = s.hd() in
  (x, s)

let rec iter f s =
  match head_opt s with 
  | None -> ()
  | Some v -> f v; iter f s

let map f s =
  { hd = (fun () -> f (s.hd())) }


let rec fold_left f accu s =
  match head_opt s with
  | None -> accu
  | Some v -> fold_left f (f accu v) s


let of_fun f = { hd = f }

let rec of_list l =
  match l with
  | [] -> snil
  | x :: xs -> scons x (of_list xs)


let to_list s =
  let l = fold_left (fun accu x -> x::accu) [] s in
  List.rev l


let of_vect v = of_list (Array.to_list v)

let to_vect s = Array.of_list (to_list s)

let synchronize (s1, s2) =
  def j1(x1) & j2(x2) & wait() = reply (x1, x2) to wait 
  in
  let f () =
    spawn j1 (head_opt s1);
    spawn j2 (head_opt s2);
    wait()
  in
  { hd = f }


let flatten s =
  let state = ref None in 
  let rec f () =
    match !state with
    | None ->
	state := Some (s.hd());
	f()
    | Some (Some x, r) -> 
	state := Some(None, r);
	x
    | Some (None, Some x) ->
	state := None;
	x
    | Some (None, None) ->
	raise End_of_stream
  in
  { hd = f }

let join (s1, s2) =
  flatten (synchronize (s1, s2))

let fork s =
  def state(Some (Some x), r) & fst() = 
     state(None, r) & reply x to fst

   or state(Some None, r) & fst() = 
     state(Some None, r) & reply raise End_of_stream to fst

   or state(l, Some (Some x)) & snd() = 
     state(l, None) & reply x to snd

   or state(l, Some None) & snd() = 
     state(l, Some None) & reply raise End_of_stream to snd

   or state(None, None) = 
     let x1 = head_opt s in
     let x2 = head_opt s in
     state(Some x1, Some x2)
  in
  spawn state(None, None);
  ({ hd = fst }, { hd = snd })

type lr = Left | Right

let fork s =
  def state(lr, Some (Some x), r) & fst() = 
     state(lr, None, r) & reply x to fst

   or state(lr, Some None, r) & fst() = 
     state(lr, Some None, r) & reply raise End_of_stream to fst

   or state(lr, l, Some (Some x)) & snd() = 
     state(lr, l, None) & reply x to snd

   or state(lr, l, Some None) & snd() = 
     state(lr, l, Some None) & reply raise End_of_stream to snd

   or state(Left, None, r) = 
     let l = head_opt s in
     state(Right, Some l, r)

   or state(Right, l, None) = 
     let r = head_opt s in
     state(Left, l, Some r)

  in
  spawn state(Left, None, None);
  ({ hd = fst }, { hd = snd })

(* XXXXX Pb dans le compilo XXXXXXXXXXX *)
(*
let fork s =
  def state(lr, (lxs, ((Some y)::lys)), rbuf) & fst() =
     state(lr, (lxs,lys), rbuf) & reply y to fst

   or state(lr, (([], [None]) as lbuf), rbuf) & fst() =
     state(lr, lbuf, rbuf) & reply raise End_of_stream to fst

   or state(lr, ((_::_, None::_) | (_, None::_::_)), rbuf) & fst() =
     reply assert false to fst (* XXX a tester XXX *)

   or state(lr, lbuf, (rxs, (Some y)::rys)) & snd() =
     state(lr, lbuf, (rxs,rys)) & reply y to snd

   or state(lr, lbuf, (([], [None]) as rbuf)) & snd() =
     state(lr, lbuf, rbuf) & reply raise End_of_stream to snd

   or state(lr, lbuf, ((_::_, None::_) | (_, None::_::_))) & snd() =
     reply assert false to snd (* XXX a tester XXX *)

   or state(Left, ([],[]), rbuf) =
     let l = head_opt s in
     state(Right, ([l],[]), rbuf)

   or state(Right, ([],[]), (rxs,rys)) =
     let r = head_opt s in
     state(Left, ([],[]), (r::rxs,rys))

   or state(Right, lbuf, ([],[])) =
     let r = head_opt s in
     state(Left, lbuf, ([r],[]))

   or state(Left, (lxs,lys), ([],[])) =
     let l = head_opt s in
     state(Right, (l::lxs,lys), ([],[]))

   or state(lr, (_::_ as lxs,[]), rbuf) & fst() =
     state(lr, ([], List.rev lxs), rbuf) & reply fst() to fst

   or state(lr, lbuf, (_::_ as rxs,[])) & snd() =
     state(lr, lbuf, ([], List.rev rxs)) & reply snd() to snd

  in
  spawn state(Left, ([],[]), ([],[]));
  ({ hd = fst }, { hd = snd })
*)
