type 'a stream = {  hd : unit -> 'a }

type 'a t = 'a stream

exception End_of_stream

let snil = { hd = (fun () -> raise End_of_stream) } 


let scons x s =
  let body =  ref (fun () -> assert false) in
  let s_hd = s.hd in
  let f () =
    body := s_hd;
    x
  in
  body := f;
  { hd = (fun () -> !body ())}


let singleton x = scons x snil

let head s = s.hd()
let head_opt s = 
  try Some (s.hd())
  with End_of_stream -> None 

let rec iter f s =
  match head_opt s with 
  | None -> ()
  | Some v -> f v; iter f s

let map f s =
  { hd = (fun () -> f (s.hd())) }


let rec fold_left f accu s =
  match head_opt s with
  | None -> accu
  | Some v -> fold_left f (f accu v) s


let of_fun f = { hd = f }

let rec of_list l =
  match l with
  | [] -> snil
  | x :: xs -> scons x (of_list xs)


let to_list s =
  let l = fold_left (fun accu x -> x::accu) [] s in
  List.rev l


let of_vect v = of_list (Array.to_list v)

let to_vect s = Array.of_list (to_list s)

let synchronize (s1, s2) =
  def j1(x1) & j2(x2) & wait() = reply (x1, x2) to wait 
  in
  let f () =
    spawn j1 (head_opt s1);
    spawn j2 (head_opt s2);
    wait()
  in
  { hd = f }


let flatten s =
  let state = ref None in 
  let rec f () =
    match !state with
    | None ->
	state := Some (s.hd());
	f()
    | Some (Some x, r) -> 
	state := Some(None, r);
	x
    | Some (None, Some x) ->
	state := None;
	x
    | Some (None, None) ->
	raise End_of_stream
  in
  { hd = f }

let join (s1, s2) =
  flatten (synchronize (s1, s2))

let fork s =
  def state(Some (Some x), r) & fst() = 
     state(None, r) & reply x to fst

   or state(Some None, r) & fst() = 
     state(Some None, r) & reply raise End_of_stream to fst

   or state(l, Some (Some x)) & snd() = 
     state(l, None) & reply x to snd

   or state(l, Some None) & snd() = 
     state(l, Some None) & reply raise End_of_stream to snd

   or state(None, None) = 
     let x1 = head_opt s in
     let x2 = head_opt s in
     state(Some x1, Some x2)
  in
  spawn state(None, None);
  ({ hd = fst }, { hd = snd })

