(* $Id: seq.mli,v 1.6 2007/09/03 13:17:52 fclement Exp $ *)

type ('a, 'b) skel
(** A skeleton is a function from lazy stream of A to lazy stream of B. *)

val seq : ('a -> 'b) -> ('a, 'b) skel;;
(** Lift a regular function from A to B to a skeleton (a function from lazy
    stream of A to lazy stream of B).
    [seq] is equivalent to [Streams.map]. *)


(*
val unseq : ('a, 'b) skel -> ('a -> 'b);;
(** Unlift a skeleton (a function from lazy stream of A to lazy stream of B)
    to a regular function from A to B. *)
*)

val pipeline : ('a, 'b) skel -> ('b, 'c) skel -> ('a, 'c) skel;;
(** [pipeline skl1 skl2] applies [skl1] then [skl2]. *)

val ( ||| ) :  ('a, 'b) skel -> ('b, 'c) skel -> ('a, 'c) skel;;
(** An infix alias for [pipeline]. *)

val farm : ('a, 'b) skel * int -> ('a, 'b) skel;;
(** Given a stream processing function [skl] and a positive integer [n],
   [farm (skl, n)] computes [skl] on each element of the input stream in
   parallel with [n] processes. If the input stream is [x1; x2; x3; ...; xk],
   the output stream is [f x1; f x2; f x3; ...; f xk] where [f = unseq skl].
   Rq: \forall skl, n>0, s, farm (skl, n) s = skl s.
   Hence, a minimal specification would be:
   let farm (skl, n) = skl;;
   which is a semantically correct implementation for farm. *)

val mapvector : ('a, 'b) skel * int -> ('a array, 'b array) skel;;
(** Given a stream processing function [skl] that maps ['a] elements of its
  input stream to ['b] elements in its output stream, given an integer [n], the
  call [mapvector (skl, n)] returns a stream processing function that maps ['a]
  vector elements of its input stream to ['b] vectors in its output stream. *)


val loop : ('a -> bool) * ('a, 'a) skel -> ('a, 'a) skel;;
(** Given a predicate function [p] and a stream processing function [skl],
    [loop (p, skl)] returns the stream processing function that loops on each
    element of the input stream until predicate [p] holds.
    In short, for each element [x] of the input stream, [loop (p, skl)] computes
    [f^n (x)] for the least integer [n] strictly greater than 1 such that
    [p (f^n (x))] holds, where [f = unseq skl]. *)

(*
val reducevector : 'a -> ('a -> 'a -> 'a) * int -> ('a array, 'a) skel;;
(** [reducevector e (f, n) sv] computes foreach element [v] of [sv], the value
  [f e (f v.(0) (f v.(1) (f v.(2) (f ... ))))].
  [f] must be a binary associative commutative operator, and [e] must be its
  corresponding neutral element. *)
*)

val parfun : ('a, 'b) skel -> 'a Streams.t -> 'b Streams.t;;

val pardo : (unit -> unit) -> unit;;
