(***********************************************************************)
(*                                                                     *)
(*                          CamlP3l                                    *)
(*                                                                     *)
(* (C) 2004-2007                                                       *)
(*             Roberto Di Cosmo (dicosmo@dicosmo.org)                  *)
(*             Zheng Li (li@pps.jussieu.fr)                            *)
(*             Pierre Weis (Pierre.Weis@inria.fr)                      *)
(*             Francois Clement (Francois.Clement@inria.fr)            *)
(*                                                                     *)
(* Based on original Ocaml P3L System                                  *)
(* (C) 1997 by                                                         *)
(*             Roberto Di Cosmo (dicosmo@ens.fr)                       *)
(*             Marco Danelutto (marcod@di.unipi.it)                    *)
(*             Xavier Leroy  (Xavier.Leroy@inria.fr)                   *)
(*             Susanna Pelagatti (susanna@di.unipi.it)                 *)
(*                                                                     *)
(* This program is free software; you can redistribute it and/or       *)
(* modify it under the terms of the GNU Library General Public License *)
(* as published by the Free Software Foundation; either version 2      *)
(* of the License, or (at your option) any later version.              *)
(*                                                                     *)
(* This program is distributed in the hope that it will be useful,     *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of      *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *)
(* GNU Library General Public License for more details.                *)
(*                                                                     *)
(***********************************************************************)

(* $Id: make_mandel.ml,v 1.1 2007/02/08 17:15:31 weis Exp $ *)

(* Convert an integer in the range 0..resolution into a screen color *)
let color_of c resolution =
  Pervasives.truncate ((float c /. float resolution) *. float Graphics.white);;

(* compute the color of a pixel by iterating z_{n + 1} = z_n ^ 2 + c       *)
(* row is the y-th row, initialised so that row.(x),y  are the coordinates *)
(* of the pixel (x, y)                                                     *)
let pixel_row (row, y, resolution, nx) = 
  let zr = ref 0.0 in
  let zi = ref 0.0 in
  let zrs = ref 0.0 in
  let zis = ref 0.0 in
  let d = 2.0 /. (float nx -. 1.0) in
  let coloured_row = Array.create nx (Graphics.black) in

  for x = 0 to nx - 1 do
    zr := float row.(x) *. d -. 1.0;
    zi := float y *. d -. 1.0;
    let cr = !zr in
    let ci = !zi in
    zrs := 0.0;
    zis := 0.0;
    for i = 0 to resolution - 1 do
      if not (!zrs +. !zis > 4.0)
      then 
        begin
          zrs := !zr *. !zr;
          zis := !zi *. !zi;
          zi  := 2.0 *. !zr *. !zi +. ci;
          zr  := !zrs -. !zis +. cr;
          Array.set coloured_row x (color_of i resolution);
        end;
    done
  done;
  (coloured_row, y);;

(* draw a row on the screen using fast image functions *)
let show_a_result (row, y) =
  Graphics.draw_image (Graphics.make_image [| row |]) 0 y;;

